Package: xts
Type: Package
Title: eXtensible Time Series
Version: 0.14.1
Authors@R: c(
  person(given=c("Jeffrey","A."), family="Ryan", role=c("aut","cph")),
  person(given=c("Joshua","M."), family="Ulrich", role=c("cre","aut"), email="josh.m.ulrich@gmail.com"),
  person(given="Ross", family="Bennett", role="ctb"),
  person(given="Corwin", family="Joy", role="ctb")
  )
Depends: R (>= 3.6.0), zoo (>= 1.7-12)
Imports: methods
LinkingTo: zoo
Suggests: timeSeries, timeDate, tseries, chron, tinytest
LazyLoad: yes
Description: Provide for uniform handling of R's different time-based data classes by extending zoo, maximizing native format information preservation and allowing for user level customization and extension, while simplifying cross-class interoperability.
License: GPL (>= 2)
URL: https://joshuaulrich.github.io/xts/,
        https://github.com/joshuaulrich/xts
BugReports: https://github.com/joshuaulrich/xts/issues
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-10-15 15:25:09 UTC; josh
Author: Jeffrey A. Ryan [aut, cph],
  Joshua M. Ulrich [cre, aut],
  Ross Bennett [ctb],
  Corwin Joy [ctb]
Maintainer: Joshua M. Ulrich <josh.m.ulrich@gmail.com>
Repository: CRAN
Date/Publication: 2024-10-15 17:30:02 UTC
Built: R 4.6.0; x86_64--netbsd; 2026-06-17 08:53:26 UTC; unix
