Package: xts
Type: Package
Title: eXtensible Time Series
Version: 0.9-7
Date: 2013-06-26
Author: Jeffrey A. Ryan, Joshua M. Ulrich
Depends: zoo (>= 1.7-10)
Suggests: timeSeries,timeDate,tseries,its,chron,fts,tis
LazyLoad: yes
Maintainer: Jeffrey A. Ryan <jeff.a.ryan@gmail.com>
Description: Provide for uniform handling of R's different time-based data classes by extending zoo, maximizing native format information preservation and allowing for user level customization and extension, while simplifying cross-class interoperability.
License: GPL (>= 2)
URL: http://r-forge.r-project.org/projects/xts/
Packaged: 2014-01-02 18:00:13 UTC; ripley
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2014-01-02 19:18:28
Built: R 3.4.1; x86_64-w64-mingw32; 2017-08-19 13:56:11 UTC; windows
Archs: x64
